@@ -11,6 +11,7 @@
 #include <storage.h>
 #include <bdd.h>
 #include <node.h>
+#include "y.tab.h"
 
 #define MIN_NODES 10000
 
@@ -18,7 +19,8 @@ extern int KEYTABLESIZE, APPLY_CACHE_SIZE, MINI_CACHE_SIZE;
 extern int nstvars,real_nstvars;
 
 #ifdef REORDER
-extern int reorder, reorder_bits, reorder_size, reorder_maxsize;
+extern int reorder, reorder_bits, reorder_size,
+  reorder_maxsize, reorder_minsize;
 extern float reorder_factor;
 #endif
 #ifdef OTHER_SIMP
@@ -42,6 +44,10 @@ static apply_rec *apply_cache;
 static int apply_cache_size;
 extern int verbose;
 
+/* Timers for reordering */
+clock_t reorder_variables_utime = 0,
+  reorder_variables_stime = 0;
+
 bdd_ptr ONE,ZERO;
 
 int             primes[] = {
@@ -530,26 +536,50 @@ set_variable_names()
   for(i = 0; i<max_level; i++)
     variable_names[i] = NIL;
     
-  for(l = variables; l; l = cdr(l))
-    { bdd_ptr f = find_assoc_bdd_var(car(l));
-      variable_names[GETLEVEL(f)] = car(l);
+  for(l = variables; l; l = cdr(l)) {
+    bdd_ptr f = find_assoc_bdd_var(car(l));
+    int level = GETLEVEL(f);
+    if(level >= NLEVELS) {
+      char str[256];
+      sprintf(str, "set_variable_names: level = %d > NLEVELS = %d",
+	      level, NLEVELS);
+      catastrophe(str);
     }
+    variable_names[level] = car(l);
+  }
 }
 
 static int last_reorder_size = 0;
 
+node_ptr variables_last_reorder = NIL;
+
 #define max(a, b) (((a) > (b))? (a) : (b))
 
 void reorder_variables()
 { 
-  int selected_level;
+  int selected_level = 1;
   int max_width;
   int i, j, k;
   int nvar = 0;
   bdd_ptr p;
+  int first_time = (variables_last_reorder == NIL);
+
+  /* The list of new variables since the last reorder.
+     We will try to reorder them first, then proceed with the rest */
+  node_ptr new_variables=NIL;
 
   int hash_size = reduce_table.n;
 
+  /* Measure the time spent for reordering */
+  struct tms buffer;
+  /* Remember the times at the moment we enter */
+  clock_t tms_utime, tms_stime;
+
+  times(&buffer);
+  tms_utime = buffer.tms_utime;  
+  tms_stime = buffer.tms_stime;
+
+  /* See if we need to reorder.  If not, just exit, do not update the timer */
   if((reduce_table.elements_in_table < 5 * last_reorder_size/4)
      || (reduce_table.elements_in_table > reorder_maxsize))
     return;
@@ -575,20 +605,77 @@ void reorder_variables()
 	n_bdd_at_level[i] = 0;
       }
 
+  /* Compute the list of new variables */
+  for(i=0; i<max_level; i++) {
+    if(variable_names[i] && n_bdd_at_level[i] > 4) {
+      node_ptr p = variables_last_reorder, tmp;
+      /* See if the var is in the old list */
+      while(p != NIL && car(p) != variable_names[i]) p = cdr(p);
+      if(p == NIL) {
+	/* var is new.  Add it to the new list. */
+	new_variables = cons(variable_names[i], new_variables);
+      }
+    }
+  }
 
-  while(selected_level >=0)
-    { selected_level = -1;
+  /* Add the new vars to the list of old vars */
+  {
+    node_ptr p = new_variables;
+    while(p != NIL) {
+      variables_last_reorder =
+	cons(car(p), variables_last_reorder);
+      p = cdr(p);
+    }
+  }
+
+  while(selected_level >=0
+	/* Stop reordering when BDDs get small enough, but reorder
+	   all the new vars. */
+	&& (reduce_table.elements_in_table >= reorder_minsize
+	    || (new_variables != NIL))
+	/* The user may toggle this option at run-time.
+           Check after placing every variable. */
+	&& reorder)
+    {
+      int is_new_var = 0;
+
+      selected_level = -1;
       max_width = 4;
-      for(i = 0; i<max_level; i++)
-	if(n_bdd_at_level[i] > max_width && variable_names[i])
-	  { selected_level = i;
+      if(new_variables != NIL) {
+	node_ptr tmp=new_variables;
+	while(tmp != NIL) {
+	  bdd_ptr bdd_var = find_assoc_bdd_var(car(tmp));
+	  int i = GETLEVEL(bdd_var);
+	  if(n_bdd_at_level[i] > max_width && variable_names[i]) {
+	    selected_level = i;
+	    max_width = n_bdd_at_level[i];
+	    is_new_var = 1;
+	  }
+	  tmp = cdr(tmp);
+	}
+      }
+      /* If no new var was selected */
+      if(selected_level < 0) {
+	/* First, erase the new vars, so we don't have 
+	   to go through them again */
+	if(new_variables != NIL) {
+	  free_list(new_variables);
+	  new_variables = NIL;
+	}
+	/* And select the next var by the max width */
+	for(i = 0; i<max_level; i++)
+	  if(n_bdd_at_level[i] > max_width && variable_names[i]) {
+	    selected_level = i;
 	    max_width = n_bdd_at_level[i];
 	  }
+      }
+      /* Still no var?  Well, then we're done. */
       if(selected_level < 0)
 	break;
       if(verbose)
-	fprintf(stderr, " %2dth var, position %3d, width %3d, ", 
-	       nvar++, selected_level, max_width);
+	fprintf(stderr, " %2dth var%s, position %3d, width %3d, ", 
+		nvar++, (is_new_var)? " (new)" : "",
+		selected_level, max_width);
       n_bdd_at_level[selected_level] = 0;
       find_optimal_position(selected_level);
 #ifdef OTHER_SIMP
@@ -630,6 +717,10 @@ void reorder_variables()
   for(p = constant_bdd_nodes; p ; p = add_to_hash_table(p))
     ;
 
+  /* Update the timers for this function */
+  times(&buffer);
+  reorder_variables_utime += buffer.tms_utime - tms_utime;
+  reorder_variables_stime += buffer.tms_stime - tms_stime;
 }
 
 
