@@ -12,6 +12,10 @@ int option_print_node_length=40;
 int option_print_width = 79;
 int interactive_mode = 0;
 int conj_part_limit = 0;
+
+/* Whether to reorder when everything is done */
+int option_final_reorder = 0;
+
 double fudge_factor;
 char *output_order_file = 0,*input_order_file = 0;
 char *input_file = "stdin";
@@ -24,7 +28,8 @@ extern node_ptr parse_tree;
 #ifdef REORDER
 int reorder = 0;
 int reorder_bits = 10;
-int reorder_size = 5000;
+int reorder_size = 10000;
+int reorder_minsize = 2000;
 int reorder_maxsize = 300000;
 float reorder_factor = 1.25;
 #endif
@@ -56,6 +61,10 @@ extern unsigned apply_cache_hit_count;
 
 int indent_size = 0;
 
+/* Timers for the variable reordering */
+extern clock_t reorder_variables_utime,
+  reorder_variables_stime;
+
 /* Functions */
 
 void init_eval(){}
@@ -86,7 +95,8 @@ int sig;
     exit(1);
   }
   for(i = 0 ; i < 32 ; i++)
-    signal(i, signal_handler);
+    if((i != SIGSEGV) && (i != SIGCONT) && (i != SIGTSTP))
+      signal(i, signal_handler);
 }
 
 void open_input(filename)
@@ -199,6 +209,14 @@ void print_usage()
   printf("user time: %g s, system time: %g s\n",
 	 buffer.tms_utime/(double)CLK_TCK,
 	 buffer.tms_stime/(double)CLK_TCK);
+  if(reorder) {
+    printf("Dynamic variable reordering: user time: %g s, system time: %g s\n",
+	   reorder_variables_utime/(double)CLK_TCK,
+	   reorder_variables_stime/(double)CLK_TCK);
+    printf("Time without reordering: user time: %g s, system time: %g s\n",
+	   (buffer.tms_utime-reorder_variables_utime)/(double)CLK_TCK,
+	   (buffer.tms_stime-reorder_variables_stime)/(double)CLK_TCK);
+  }
   printf("BDD nodes allocated: %d\n",get_bdd_nodes_allocated());
   printf("Bytes allocated: %d\n",(unsigned)((char *)sbrk(0)-addrstart));
 #ifdef SERGEYDEBUG
@@ -222,6 +240,9 @@ void catastrophe(s,a1,a2,a3,a4)
 char *s,*a1,*a2,*a3,*a4;
 {
   fprintf(stderr,"\n\n*** internal error *** ");
+#ifdef VERSION
+  fprintf(stderr,"\n%s", VERSION);
+#endif
   fprintf(stderr,s,a1,a2,a3,a4);
   fprintf(stderr,"\nPlease report this error to sergey.berezin@cs.cmu.edu\n");
   fprintf(stderr,"Send a copy of this output and your input.\n");
