@@ -22,6 +22,7 @@
 #include <y.tab.h>
 
 extern int option_print_node_length;
+extern int option_print_width;
 
 static mgr_ptr node_mgr;
 static hash_ptr node_hash;
@@ -102,12 +103,16 @@ node_ptr x,y;
 node_ptr car(x)
 node_ptr x;
 {
+/*    if(x->type == NUMBER) */
+/*      catastrophe("car: node type is NUMBER"); */
   return(x->left.nodetype);
 }
 
 node_ptr cdr(x)
 node_ptr x;
 {
+/*    if(x->type == NUMBER) */
+/*      catastrophe("car: node type is NUMBER"); */
   return(x->right.nodetype);
 }
 
@@ -248,14 +253,14 @@ int p;
   case ATOM:
     if(!my_strncat(str,n->left.strtype->text,size))return(0);
     if(cdr(n)){
-      char buf[20];
+      char buf[ATOM_MAX_LENGTH];
       sprintf(buf,"_%d",cdr(n));
       return(my_strncat(str,buf,size));
     }
     return(1);
   case NUMBER:
     {
-      char buf[20];
+      char buf[ATOM_MAX_LENGTH];
       sprintf(buf,"%d",car(n));
       return(my_strncat(str,buf,size));
     }
@@ -274,6 +279,15 @@ int p;
 	   && my_strncat(str,"[",size)
 	   && sprint_node(str,size,n->right.nodetype)
 	   && my_strncat(str,"]",size));
+  case CASE: return(my_strncat(str,"case ",size)
+		    && sprint_node(str,size,n->left.nodetype)
+		    && my_strncat(str," 1: ",size)
+		    && sprint_node(str,size,n->right.nodetype)
+		    && my_strncat(str," esac",size));
+  case COLON: return(sprint_node(str,size,n->left.nodetype)
+		     && my_strncat(str," : ",size)
+		     && sprint_node(str,size,n->right.nodetype)
+		     && my_strncat(str,"; ",size));
   case TWODOTS: op = ".."; prio = 3; opkind = 1; break;
   case IMPLIES: op = "->"; prio = 4; opkind = 1; break;
   case IFF: op = "<->"; prio = 4; opkind = 1; break;
@@ -327,6 +341,9 @@ int p;
   case NEXT:
     if(!my_strncat(str,"next",size))return(0);
     op = ""; prio = 0; p = 1; opkind = 0; break;
+  case SMALLINIT:
+    if(!my_strncat(str,"init",size))return(0);
+    op = ""; prio = 0; p = 1; opkind = 0; break;
   default:
     catastrophe("sprint_node1: type = %d",n->type);
   }
@@ -450,6 +467,19 @@ int p;
     fprint_node(ff,n->right.nodetype);
     fprintf(ff,"]");
     return;
+  case CASE: 
+    fprintf(ff,"case ");
+    fprint_node(ff,n->left.nodetype);
+    fprintf(ff," 1: ");
+    fprint_node(ff,n->right.nodetype);
+    fprintf(ff," esac");
+    return;
+  case COLON: 
+    fprint_node(ff,n->left.nodetype);
+    fprintf(ff," : ");
+    fprint_node(ff,n->right.nodetype);
+    fprintf(ff,"; ");
+    return;
   case HIDE:
   case EXPOSE:
     {
@@ -519,6 +549,9 @@ int p;
   case NEXT:
     fprintf(ff,"next");
     op = ""; prio = 0; p = 1; opkind = 0; break;
+  case SMALLINIT:
+    fprintf(ff,"init");
+    op = ""; prio = 0; p = 1; opkind = 0; break;
   default:
     catastrophe("fprint_node1: type = %d",n->type);
   }
@@ -576,19 +609,21 @@ FILE *stream;
 node_ptr n;
 int col;
 {
-  char buf[41];
+  char *buf = (char *)malloc(option_print_node_length + 1);
   int c,p;
+  if(buf == NULL) rpterr("Out of memory");
   buf[0] = 0;
   c = sprint_node(buf,option_print_node_length,n);
   p = strlen(buf);
   if(!c) p += 3;
-  if(col + p >= 79){
+  if(col + p >= option_print_width){
     fprintf(stream,"\n");
     col = 0;
     while((col++) < indent_size + 1)fprintf(stream,"  ");
   }
   fprintf(stream,"%s",buf);
   if(!c)fprintf(stream,"...");
+  free(buf);
   return(col + p);
 }
 
