@@ -149,14 +149,14 @@
 (defconst smv-keywords
   '("MODULE" "VAR" "TRANS" "ASSIGN" "INVAR" "DEFINE" "SPEC" "process" "array"
     "FAIRNESS" "case" "esac" "AG" "EG" "AF" "EF" "AX" "EX" "U" "of" "mod"
-    "for" "WORDS" "WORDDEFINE" "boolean" "next" "init" "INIT" "powerof2"
+    "for" "WORDS" "WORDDEFINE" "boolean" "next" "init" "INIT" "ISA" "powerof2"
     "wordconst" "toword" "self" "in" "notin" "COMPUTE" "MAX" "MIN" "PRINT"
     "hide" "expose")
   "The list of SMV keywords.")
 
 (defconst smv-declaration-keywords
   '("MODULE" "VAR" "TRANS" "ASSIGN" "INVAR" "DEFINE" "SPEC"
-    "FAIRNESS" "WORDS" "WORDDEFINE" "INIT")
+    "FAIRNESS" "WORDS" "WORDDEFINE" "INIT" "ISA")
   "The list of keywords that open a declaration. Used for indentation.")
 
 (defconst smv-declaration-keywords-regexp
@@ -302,7 +302,7 @@ This variable is updated automatically each time SMV process takes off.")
 
 (defvar smv-options-changed nil)
 
-(defun smv-args (&optional args)
+(defun smv-args (file &optional args)
   "Compiles the string of SMV command line args from various variables."
   (mapconcat 'identity
 	     (append
@@ -318,7 +318,11 @@ This variable is updated automatically each time SMV process takes off.")
 	      (if smv-order-file (list "-i" smv-order-file) nil)
 	      (if smv-verbose-level (list "-v" smv-verbose-level) nil)
 	      (if smv-heuristic-factor (list "-h" smv-heuristic-factor) nil)
-	      (if smv-command-line-args (list smv-command-line-args) nil)
+	      (if smv-command-line-args 
+		  (if (string-match "%s" smv-command-line-args)
+		      (list (format smv-command-line-args file))
+		    (list smv-command-line-args file))
+		(list file))
 	      (if args (list args) nil))
 	     " "))
 
@@ -330,8 +334,8 @@ This variable is updated automatically each time SMV process takes off.")
 	(progn
 	  (if (buffer-modified-p) (smv-save-buffer))
 	  (setq smv-compile-buffer 
-		(compile-internal (concat smv-command " " (smv-args) " "
-					  (buffer-file-name))
+		(compile-internal
+		 (concat smv-command " " (smv-args (buffer-file-name)))
 				  "No more errors"
 				  "SMV"))
 	  (set-buffer smv-compile-buffer) ;;; Doesn't work???
@@ -345,12 +349,12 @@ This variable is updated automatically each time SMV process takes off.")
   (interactive)
   (if (buffer-file-name)
       (let* ((buffer (current-buffer))
-	     (buffer-file (buffer-file-name))
-	     (match (string-match "\\.smv$" (buffer-file-name)))
+	     (buffer-file (file-name-nondirectory (buffer-file-name)))
+	     (match (string-match "\\.smv$" buffer-file))
 	     (order-file-name (if match
-				 (concat (substring (buffer-file-name) 0 match)
+				 (concat (substring buffer-file 0 match)
 					 ".ord")
-				(concat (buffer-file-name) ".ord")))
+				(concat buffer-file ".ord")))
 	     (do-it (if (file-exists-p order-file-name)
 			(y-or-n-p (format "The file %s already exists. Overwrite it?" order-file-name))
 		      t)))
@@ -386,14 +390,15 @@ no such file, generate it and load (see `smv-generate-order-file')."
   (interactive)
   (setq smv-current-buffer (current-buffer))
   (if (or smv-order-file (buffer-file-name))
-      (let* ((order-file-name (or smv-order-file
+      (let* ((buffer-file (file-name-nondirectory (buffer-file-name)))
+	     (order-file-name (or smv-order-file
 				  (let ((match (string-match "\\.smv$"
-						       (buffer-file-name))))
+						       buffer-file)))
 				    (if match
-					(concat (substring (buffer-file-name)
+					(concat (substring buffer-file
 							   0 match)
 						".ord")
-				      (concat (buffer-file-name) ".ord"))))))
+				      (concat buffer-file ".ord"))))))
 	(setq smv-order-file order-file-name)
 	(if (file-exists-p order-file-name)
 	    (find-file-other-window order-file-name)
